//-	---------------------------------------------------------------------------
//-	Name: 		SetColumn.cpp
//-
//-	Purpose:	This file provides the implementation of the class
//-				CSetColumn.
//-
//-	Revisions:
//-	Date		Author		Change Summary
//-	---------	------		---------------------------------------------------
//-	09-30-99	Mklombardi@ Created & Released.
//-             netzero,net 
//-
//-	Notes:		None.
//-
//- Copyright : This software is NOT copyrighted.
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in  this file is used in any commercial application 
// then acknowledgement must be made to the author of this file 
// (in whatever form you wish).
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability for any damage/loss of business that
// this product may cause.
//
// Expect bugs!
// 
// Please use and enjoy. Please let me know of any bugs/mods/improvements 
// that you have found/implemented and I will fix/incorporate them into this
// file. 
//
//-	---------------------------------------------------------------------------
// SetColumn.cpp : implementation file
//

#include "stdafx.h"
#include "SDIGridCtl.h"
#include "SetColumn.h"
#include "DropListData.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define Slash 0x5C
#define LF    0x0A
#define Lower_n 0x6e

/////////////////////////////////////////////////////////////////////////////
// CSetColumn dialog


CSetColumn::CSetColumn(CWnd* pParent /*=NULL*/)
	: CDialog(CSetColumn::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSetColumn)
	m_ColumnName = _T("");
	m_ColumnId = 0;
	m_ColWidth = DefaultColWidth;
	//}}AFX_DATA_INIT
    m_NumCols = 0;
}


void CSetColumn::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSetColumn)
	DDX_Control(pDX, IDC_DROP_LIST_ITEMS, m_DropListItemsButton);
	DDX_Control(pDX, IDC_COLUMN_NAME, m_ColCtl);
	DDX_Control(pDX, IDC_LISTBOX, m_Listbox);
	DDX_Control(pDX, IDC_NEXT_BUTTON, m_NextButton);
	DDX_Control(pDX, IDC_PREVIOUS_BUTTON, m_PreviousButton);
	DDX_Text(pDX, IDC_COLUMN_NAME, m_ColumnName);
	DDX_Text(pDX, IDC_WHICH_COLUMN, m_ColumnId);
	DDV_MinMaxInt(pDX, m_ColumnId, 0, 50);
	DDX_Text(pDX, IDC_COL_WIDTH, m_ColWidth);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSetColumn, CDialog)
	//{{AFX_MSG_MAP(CSetColumn)
	ON_BN_CLICKED(IDC_NEXT_BUTTON, OnNextButton)
	ON_BN_CLICKED(IDC_PREVIOUS_BUTTON, OnPreviousButton)
	ON_WM_TIMER()
	ON_LBN_SELCHANGE(IDC_LISTBOX, OnSelchangeListbox)
	ON_BN_CLICKED(IDC_DROP_LIST_ITEMS, OnDropListItems)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSetColumn message handlers

CSetColumn::~CSetColumn()
{
}

BOOL CSetColumn::OnInitDialog() 
{
	CDialog::OnInitDialog();

    m_Listbox.AddString(_T("Editable"));
    m_Listbox.AddString(_T("Checkbox"));
    m_Listbox.AddString(_T("Listbox"));
    m_Listbox.SetCurSel(0);
    GetData();
    SetTimer(233, 100, NULL);
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CSetColumn::GetData()
{
    POSITION CurrentPosition = m_pItems->FindIndex(m_ColumnId);
    if (CurrentPosition != NULL)
    {
        CellDataType* pCellData = (CellDataType *)m_pItems->GetAt(CurrentPosition);

        //if (New cell)
        if (pCellData == NULL)
        {
            m_ColumnName.Empty();
            m_Listbox.SetCurSel(0);
        	m_ColWidth = DefaultColWidth;
        }
        else                            
        {   //Extract existing record;
            m_ColumnName = pCellData->Desc;
            m_ColumnName = InsertEscapeCodes(m_ColumnName);
            m_Listbox.SetCurSel(pCellData->ItemType);
            m_ColWidth = pCellData->ColWidth;

            if (pCellData->ItemType == ListData)
            {
                m_DropListData.Empty();
                for (int Index = 0; Index < pCellData->DropListItems.GetSize(); Index++)
                {
                    m_DropListData += (CString)pCellData->DropListItems.GetAt(Index);
                    m_DropListData += CRLF;
                }
            }
        }
    }
    else
    {
        m_ColumnName.Empty();
    	m_ColWidth = DefaultColWidth;
        m_Listbox.SetCurSel(0);
    }

    OnSelchangeListbox();
    UpdateData(FALSE);
}

void CSetColumn::StoreData(BOOLEAN NewElement)
{
    UpdateData(TRUE);

    m_ColumnName = ConvertEscapeCodes(m_ColumnName);

    CString WorkingDropListData = m_DropListData;

    CellDataType* pNewCellData;
    pNewCellData = new CellDataType;
    pNewCellData->Id = m_ColumnId;
    pNewCellData->Desc = m_ColumnName;
    pNewCellData->ColWidth  = m_ColWidth;

    pNewCellData->ItemType = (ItemDataType)m_Listbox.GetCurSel();

    if (pNewCellData->ItemType == ListData)
    {
        int Spacer = WorkingDropListData.Find(CRLF);

        while ((Spacer != -1) && (!WorkingDropListData.IsEmpty()))
        {
            //Strip out each line from the CString;
            CString DropListEntry = WorkingDropListData.Left(Spacer);
            if (!DropListEntry.IsEmpty())
                pNewCellData->DropListItems.Add(DropListEntry);
            WorkingDropListData = WorkingDropListData.Right(WorkingDropListData.GetLength() - Spacer - 2);
            Spacer = WorkingDropListData.Find(CRLF);
        }
    }

    POSITION CurrentPosition = m_pItems->FindIndex(m_ColumnId);
    if (CurrentPosition != NULL)
    {
        CellDataType* pCellData = (CellDataType *)m_pItems->GetAt(CurrentPosition);

        //if (New cell)
        if (pCellData == NULL)
        {
            m_pItems->AddTail((CObject *)pNewCellData);
        }
        else
        {   
            //Update existing record;
            pCellData->Desc = m_ColumnName;
            pCellData->ItemType = (ItemDataType)m_Listbox.GetCurSel();
            pCellData->ColWidth  = m_ColWidth;

            pCellData->DropListItems.RemoveAll();

            int NumItems = pNewCellData->DropListItems.GetSize();

            for (int Index = 0; Index < NumItems; Index++)
            {
                CString EachItem = pNewCellData->DropListItems.GetAt(Index);
                pCellData->DropListItems.Add(EachItem);
            }

            //Do not need new cell;
            delete pNewCellData;
        }
    }
    else
        m_pItems->AddTail((CObject *)pNewCellData);

    if (NewElement)
    {
        m_ColumnName.Empty();
        m_Listbox.SetCurSel(0);
    }

    OnSelchangeListbox();
}

void CSetColumn::OnNextButton() 
{
    if (m_Listbox.GetCurSel() == LB_ERR)
    {
        AfxMessageBox("Select a column");
        return;
    }

    StoreData(TRUE);

	if (m_ColumnId == 0)
        m_PreviousButton.EnableWindow(TRUE);

    m_ColumnId++;

    if (m_ColumnId == m_NumCols-1)
        m_NextButton.EnableWindow(FALSE);

    GetData();
    SetTimer(233, 100, NULL);
}

void CSetColumn::OnPreviousButton() 
{    
    if (m_Listbox.GetCurSel() == LB_ERR)
    {
        AfxMessageBox("ERROR SELECT COLUMN");
        return;
    }

    StoreData(TRUE);

    if (m_ColumnId == m_NumCols-1)
        m_NextButton.EnableWindow(TRUE);

    m_ColumnId--;

	if (m_ColumnId == 0)
        m_PreviousButton.EnableWindow(FALSE);

    GetData();
    SetTimer(233, 100, NULL);
}

void CSetColumn::SetHeadings(CObList *pExistingItems)
{
    m_pItems = pExistingItems;
}

void CSetColumn::OnOK() 
{
    //Save latest;
	StoreData(TRUE);
	CDialog::OnOK();
}

void CSetColumn::OnTimer(UINT nIDEvent) 
{
    KillTimer(nIDEvent);
    m_ColCtl.SetFocus();
	CDialog::OnTimer(nIDEvent);
}

void CSetColumn::OnSelchangeListbox()
{
    //Enable button if listbox data selected;
    m_DropListItemsButton.EnableWindow(m_Listbox.GetCurSel() == ListData);
}

void CSetColumn::OnDropListItems() 
{
    CString ExistingDropListData;
    CDropListData DropListDataDlg;

    //Get current column we are editing;
    POSITION CurrentPosition = m_pItems->FindIndex(m_ColumnId);
    if (CurrentPosition != NULL)
    {
        CellDataType* pCellData = (CellDataType *)m_pItems->GetAt(CurrentPosition);

        //if (Existing cell)
        if (pCellData != NULL)
        {
            //How many items in the drop list;
            int NumDropListItems = pCellData->DropListItems.GetSize();

            //Reset CString;
            ExistingDropListData.Empty();

            for (int Index = 0; Index < NumDropListItems; Index++)
            {
                ExistingDropListData += (CString)pCellData->DropListItems.GetAt(Index);
                ExistingDropListData += CRLF;
            }

            //Set existing drop list data;
            DropListDataDlg.SetDropListData(ExistingDropListData);
        }

        //Invoke drop list data dialog;
        if (DropListDataDlg.DoModal() == IDOK)
        {
            //Update drop list data;
            m_DropListData = DropListDataDlg.GetDropListData();
            m_DropListData += CRLF;

            //Save this new drop list data in the list;
            StoreData(FALSE);
        }
    }
}

CString CSetColumn::ConvertEscapeCodes(CString InputString)
{
    CString ReturnString = InputString;

    if (!InputString.IsEmpty())
    {
        int Location = InputString.Find(Slash);

        while (Location != -1)
        {
            CString LeftPart = InputString.Left(Location);
            CString RightPart = InputString.Right(InputString.GetLength() - Location-2);
            ReturnString = LeftPart;
            ReturnString += LF;
            ReturnString += RightPart;
            Location = ReturnString.Find(Slash);
        }
    }

    return ReturnString;
}

CString CSetColumn::InsertEscapeCodes(CString InputString)
{
    CString ReturnString = InputString;

    if (!InputString.IsEmpty())
    {
        int Location = InputString.Find(LF);

        while (Location != -1)
        {
            CString LeftPart = InputString.Left(Location);
            CString RightPart = InputString.Right(InputString.GetLength() - Location-1);
            ReturnString = LeftPart;
            ReturnString += Slash;
            ReturnString += Lower_n;
            ReturnString += RightPart;
            Location = ReturnString.Find(LF);
        }
    }

    return ReturnString;
}